@@ -1,8 +1,17 @@
-from sqlalchemy.util import ScopedRegistry, to_list, get_cls_kwargs
-from sqlalchemy.orm import MapperExtension, EXT_CONTINUE, object_session, class_mapper
+# scoping.py
+# Copyright (C) the SQLAlchemy authors and contributors
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
+import sqlalchemy.exceptions as sa_exc
+from sqlalchemy.util import ScopedRegistry, to_list, get_cls_kwargs, deprecated
+from sqlalchemy.orm import (
+    EXT_CONTINUE, MapperExtension, class_mapper, object_session
+    )
+from sqlalchemy.orm import exc as orm_exc
 from sqlalchemy.orm.session import Session
-from sqlalchemy import exceptions
-import types
+
 
 __all__ = ['ScopedSession']
 
@@ -13,13 +22,8 @@ class ScopedSession(object):
     Usage::
 
       Session = scoped_session(sessionmaker(autoflush=True))
-      
-      To map classes so that new instances are saved in the current
-      Session automatically, as well as to provide session-aware
-      class attributes such as "query":
-      
-      mapper = Session.mapper
-      mapper(Class, table, ...)
+
+      ... use session normally.
 
     """
 
@@ -33,7 +37,7 @@ class ScopedSession(object):
             scope = kwargs.pop('scope', False)
             if scope is not None:
                 if self.registry.has():
-                    raise exceptions.InvalidRequestError("Scoped session is already present; no new arguments may be specified.")
+                    raise sa_exc.InvalidRequestError("Scoped session is already present; no new arguments may be specified.")
                 else:
                     sess = self.session_factory(**kwargs)
                     self.registry.set(sess)
@@ -42,62 +46,83 @@ class ScopedSession(object):
                 return self.session_factory(**kwargs)
         else:
             return self.registry()
-    
+
     def remove(self):
         if self.registry.has():
             self.registry().close()
         self.registry.clear()
-    
+
+    @deprecated("Session.mapper is deprecated.  "
+        "Please see http://www.sqlalchemy.org/trac/wiki/UsageRecipes/SessionAwareMapper "
+        "for information on how to replicate its behavior.")
     def mapper(self, *args, **kwargs):
-        """return a mapper() function which associates this ScopedSession with the Mapper."""
+        """return a mapper() function which associates this ScopedSession with the Mapper.
         
+        DEPRECATED.
+        
+        """
+
         from sqlalchemy.orm import mapper
-        
-        extension_args = dict([(arg,kwargs.pop(arg))
-                               for arg in get_cls_kwargs(_ScopedExt)
-                               if arg in kwargs])
-        
+
+        extension_args = dict((arg, kwargs.pop(arg))
+                              for arg in get_cls_kwargs(_ScopedExt)
+                              if arg in kwargs)
+
         kwargs['extension'] = extension = to_list(kwargs.get('extension', []))
         if extension_args:
             extension.append(self.extension.configure(**extension_args))
         else:
             extension.append(self.extension)
         return mapper(*args, **kwargs)
-        
+
     def configure(self, **kwargs):
         """reconfigure the sessionmaker used by this ScopedSession."""
-        
+
         self.session_factory.configure(**kwargs)
 
-    def query_property(self):
+    def query_property(self, query_cls=None):
         """return a class property which produces a `Query` object against the
         class when called.
-        
+
         e.g.::
             Session = scoped_session(sessionmaker())
-            
+
             class MyClass(object):
                 query = Session.query_property()
-                
+
             # after mappers are defined
             result = MyClass.query.filter(MyClass.name=='foo').all()
-        
+
+        Produces instances of the session's configured query class by
+        default.  To override and use a custom implementation, provide
+        a ``query_cls`` callable.  The callable will be invoked with
+        the class's mapper as a positional argument and a session
+        keyword argument.
+
+        There is no limit to the number of query properties placed on
+        a class.
+
         """
-        
         class query(object):
             def __get__(s, instance, owner):
-                mapper = class_mapper(owner, raiseerror=False)
-                if mapper:
-                    return self.registry().query(mapper)
-                else:
+                try:
+                    mapper = class_mapper(owner)
+                    if mapper:
+                        if query_cls:
+                            # custom query class
+                            return query_cls(mapper, session=self.registry())
+                        else:
+                            # session's configured query class
+                            return self.registry().query(mapper)
+                except orm_exc.UnmappedClassError:
                     return None
         return query()
-        
+
 def instrument(name):
     def do(self, *args, **kwargs):
         return getattr(self.registry(), name)(*args, **kwargs)
     return do
-for meth in ('add', 'add_all', 'get', 'load', 'close', 'save', 'commit', 'update', 'save_or_update', 'flush', 'query', 'delete', 'merge', 'clear', 'refresh', 'expire', 'expunge', 'rollback', 'begin', 'begin_nested', 'connection', 'execute', 'scalar', 'get_bind', 'is_modified', '__contains__', '__iter__'):
+for meth in Session.public_methods:
     setattr(ScopedSession, meth, instrument(meth))
 
 def makeprop(name):
@@ -106,30 +131,28 @@ def makeprop(name):
     def get(self):
         return getattr(self.registry(), name)
     return property(get, set)
-for prop in ('bind', 'dirty', 'deleted', 'new', 'identity_map'):
+for prop in ('bind', 'dirty', 'deleted', 'new', 'identity_map', 'is_active'):
     setattr(ScopedSession, prop, makeprop(prop))
 
 def clslevel(name):
-    def do(cls, *args,**kwargs):
+    def do(cls, *args, **kwargs):
         return getattr(Session, name)(*args, **kwargs)
     return classmethod(do)
-for prop in ('close_all','object_session', 'identity_key'):
+for prop in ('close_all', 'object_session', 'identity_key'):
     setattr(ScopedSession, prop, clslevel(prop))
-    
+
 class _ScopedExt(MapperExtension):
     def __init__(self, context, validate=False, save_on_init=True):
         self.context = context
         self.validate = validate
         self.save_on_init = save_on_init
-    
+        self.set_kwargs_on_init = True
+
     def validating(self):
         return _ScopedExt(self.context, validate=True)
-    
+
     def configure(self, **kwargs):
         return _ScopedExt(self.context, **kwargs)
-    
-    def get_session(self):
-        return self.context.registry()
 
     def instrument_class(self, mapper, class_):
         class query(object):
@@ -137,28 +160,38 @@ class _ScopedExt(MapperExtension):
                 return getattr(self.context.registry().query(class_), key)
             def __call__(s):
                 return self.context.registry().query(class_)
+            def __get__(self, instance, cls):
+                return self
 
-        if not 'query' in class_.__dict__: 
+        if not 'query' in class_.__dict__:
             class_.query = query()
-        
+
+        if self.set_kwargs_on_init and class_.__init__ is object.__init__:
+            class_.__init__ = self._default__init__(mapper)
+
+    def _default__init__(ext, mapper):
+        def __init__(self, **kwargs):
+            for key, value in kwargs.items():
+                if ext.validate:
+                    if not mapper.get_property(key, resolve_synonyms=False,
+                                               raiseerr=False):
+                        raise sa_exc.ArgumentError(
+                            "Invalid __init__ argument: '%s'" % key)
+                setattr(self, key, value)
+        return __init__
+
     def init_instance(self, mapper, class_, oldinit, instance, args, kwargs):
         if self.save_on_init:
-            entity_name = kwargs.pop('_sa_entity_name', None)
             session = kwargs.pop('_sa_session', None)
-        if not isinstance(oldinit, types.MethodType):
-            for key, value in kwargs.items():
-                if self.validate:
-                    if not mapper.get_property(key, resolve_synonyms=False, raiseerr=False):
-                        raise exceptions.ArgumentError("Invalid __init__ argument: '%s'" % key)
-                setattr(instance, key, value)
-            kwargs.clear()
-        if self.save_on_init:
-            session = session or self.context.registry()
-            session._save_impl(instance, entity_name=entity_name)
+            if session is None:
+                session = self.context.registry()
+            session._save_without_cascade(instance)
         return EXT_CONTINUE
 
     def init_failed(self, mapper, class_, oldinit, instance, args, kwargs):
-        object_session(instance).expunge(instance)
+        sess = object_session(instance)
+        if sess:
+            sess.expunge(instance)
         return EXT_CONTINUE
 
     def dispose_class(self, mapper, class_):
@@ -169,6 +202,3 @@ class _ScopedExt(MapperExtension):
                 delattr(class_, '__init__')
         if hasattr(class_, 'query'):
             delattr(class_, 'query')
-            
-
-            
